#include "glbox.h"
#include "gettext.h"
#include "get_path.h"
#include "xpm/cursor_zoom.xpm"

void GLBox::constructeur( void )
  {
   bouttonL = bouttonM = bouttonR = FALSE;
   for( int i=0 ; i<16 ; i++ ) obj[i] = NULL;
   obj_plat = NULL;
   princip = NULL;
   vue = FALSE;
   set_RAZ();
  }

GLBox::GLBox( QWidget* parent, const char* name, const QGLWidget* shareWidget )
     : QGLWidget( parent, name, shareWidget )
  {
   constructeur();
  }


GLBox::GLBox( const QGLFormat& format, QWidget* parent, const char* name, const QGLWidget* shareWidget )
     : QGLWidget( format, parent, name, shareWidget )
  {
   constructeur();
  }

void GLBox::set_theme( char *theme )
  {
   QApplication::setOverrideCursor( Qt::waitCursor );

   obj3d::set_new_theme();

   char tmp[200];

   QImage blanc, noir, plateau, buf;

   sprintf( tmp, "theme/%s/white.png", theme ); 
   if( buf.load( get_path(tmp) ) == true )
      blanc = QGLWidget::convertToGLFormat( buf );
    else
      printf( _("Error file \"%s\" not loaded\n"), tmp );

   sprintf( tmp, "theme/%s/black.png", theme ); buf.load( tmp );

   if( buf.load( get_path(tmp) ) == true )
      noir = QGLWidget::convertToGLFormat( buf );
    else
      printf( _("Error file \"%s\" not loaded\n"), tmp );

   sprintf( tmp, "theme/%s/board.png", theme );
   if( buf.load( get_path(tmp) ) == true )
      plateau = QGLWidget::convertToGLFormat( buf );
    else
      printf( _("Error file \"%s\" not loaded\n"), tmp );

   obj3d::SET_TEXTURE( blanc, noir, plateau );

   for( int i=0 ; i<16 ; i++ )
     {
      GLfloat x=0, y=0;
      if( obj[i] != NULL )
        {
         obj[i]->get_pos( x, y );
         delete obj[i];
        }
      char nom[20];
      sprintf( nom, "theme/%s/%i.piece", theme, i );
      obj[i] = new obj3d( theme, nom );
      if( x!=0 && y!=0 )
         obj[i]->set_pos( x, y );
       else
         piece_maison( i );
     }

   if( obj_plat != NULL ) delete obj_plat;
   char nom[20];
   sprintf( nom, "theme/%s/board.piece", theme );
   obj_plat = new obj3d( theme, nom );
   obj_plat->set_pos( 0, 0 );

   updateGL();

   QApplication::restoreOverrideCursor();
  }


void GLBox::set_move( principal *p )
  {
   vue = TRUE;
   princip = p;
   if( p != NULL )
     {
      place_RAZ();
      updateGL();
     }
  }

void GLBox::move_trans( bool m )
  {
   vue = m;
   if( vue == TRUE ) updateGL();
  }


GLBox::~GLBox()
  {
   for( int i=0 ; i<16 ; i++ )
      delete obj[i];
   delete obj_plat;
  }


static int init_sc = 0;
void GLBox::initializeGL()
  {
   if( init_sc != 0 ) return; else init_sc = 1;
   glClearColor( 1, 0.9, 0.7, 0.0 );
   glEnable( GL_DEPTH_TEST );
   switch_mode( 2 );
   set_theme( "modern" );

   init_sc = 0;

   GLfloat L0pos[]={ 2,-1,1,1};
   GLfloat L0dif[]={1,1,1,1};
   GLfloat L0sep[]={.1,.1,.1,0};

   glShadeModel(GL_SMOOTH);
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glLightfv(GL_LIGHT0,GL_POSITION,L0pos);
   glLightfv(GL_LIGHT0,GL_DIFFUSE,L0dif);
   glLightfv(GL_LIGHT0,GL_SPECULAR,L0sep);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(-1, 1, -1, 1, -10, 10);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
  }


void GLBox::resizeGL( int w, int h )
  {
   if( w > h )
     glViewport(0,(h-w)/2,w,w);
   else
     glViewport((w-h)/2,0,h,h);
  }


void GLBox::paintGL()
  {
   if( princip == NULL || vue == FALSE )
     {
      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
     }
   else
     {
      glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
      glPushMatrix();

      glScalef( scale, scale, scale );

      //GLfloat L0pos[]={ 2,-1,1,0};
      GLfloat L0pos[]={ 70,70,70,1};
      glLightfv(GL_LIGHT0,GL_POSITION,L0pos);

      glRotatef( xRot, 1.0, 0.0, 0.0 );
      glRotatef( yRot, 0.0, 1.0, 0.0 );
      glRotatef( zRot, 0.0, 0.0, 1.0 );

      for( int i=0 ; i<16; i++ ) obj[i]->aff( mode );
      obj_plat->aff( mode );

      glPopMatrix();
     }
  }


void GLBox::set_RAZ()
  {
   xRot = -60;
   yRot =   0;
   zRot = -25;
   scale = .15;
   updateGL();
  }


void GLBox::setXRotation( int degrees )
  {
   xRot = (GLfloat)(xRot+degrees);
   if( xRot >   0 ) xRot=  0;
   if( xRot < -90 ) xRot=-90;
   updateGL();
  }

void GLBox::setZRotation( int degrees )
  {
   zRot = (GLfloat)(zRot+degrees);
   if( zRot > 360 ) zRot-=360;
   if( zRot <   0 ) zRot+=360;
   updateGL();
  }

void GLBox::set_scale( float size )
  {
   scale += size;
   if( scale >  0.5 ) scale= .5;
   if( scale <  0.1 ) scale= 0.1;
   updateGL();
  }


void GLBox::mousePressEvent(QMouseEvent* event)
  {
   if( princip == NULL ) return;

  // LeftButton, RightButton, MidButton and NoButton.
   if( event->button() == LeftButton )
     {
      piece_depl = pick( event->x(), event->y() );
      if( piece_depl != -1 )
        {
         if( princip->appuyer( piece_depl ) )
           {
            bouttonL=true;
            setCursor( QCursor( PointingHandCursor ) );
            obj[piece_depl]->get_pos( piece_x, piece_y );
           }
        }
     }
   if( event->button() == MidButton )
     {
      bouttonM=true;
      QPixmap zoom( cursor_zoom );
      setCursor( QCursor( zoom ) );
      yM = event->y();
     }
   if( event->button() == RightButton )
     {
      bouttonR=true;
      setCursor( QCursor( SizeAllCursor ) );
      xR = event->y();
      yR = event->x();
     }
  }

void GLBox::mouseReleaseEvent(QMouseEvent* event)
  {
   if( princip == NULL ) return;

   if( event->button() == LeftButton  )
     {
      if( bouttonL )
        {
         GLfloat x, y;
         obj[piece_depl]->get_pos( x, y );
         GLint x_ = (GLint)(x+1.5+0.5+1)-1,
               y_ = (GLint)(y+1.5+0.5+1)-1;
         position p = { (x_>=0 && x_<=3)?x_:-1, (y_>=0 && y_<=3)?y_:-1 };
         if( p.x == -1 || p.y == -1 ) { p.x = -1; p.y = -1; }

         if( ! princip->lacher( p ) )
           {
            obj[piece_depl]->set_pos( piece_x, piece_y );
            updateGL();
           }
          else
           {
            if( p.x != -1 ) set_piece_pos( piece_depl, p );
           }
        }
      bouttonL=false;
     }
   if( event->button() == MidButton   ) bouttonM=false;
   if( event->button() == RightButton ) bouttonR=false;
   unsetCursor();
  }

void GLBox::mouseMoveEvent(QMouseEvent* event)
  {
   if( princip == NULL ) return;

   if( bouttonL )
     {
      GLint viewport[4];
      glGetIntegerv(GL_VIEWPORT, viewport);

      float x =  ( event->x()-viewport[0] - viewport[2]/2 );
      float y = -(float)( event->y()-viewport[1] - viewport[3]/2 )/cos((float)xRot*3.14/180);

      float t = atan2( y, x );
      float r = x / cos(t);

      t -= zRot*3.14/180;

      x = r * cos( t );
      y = r * sin( t );

      x = x/viewport[2]*2/scale;
      y = y/viewport[3]*2/scale;

      obj[piece_depl]->set_pos( x, y );

      updateGL();
     }
   if( bouttonM )
     {
      set_scale( (float)(event->y()-yM)/100 );
      yM = event->y();
     }
   if( bouttonR )
     {
      setXRotation((event->y()-xR)/2);
      setZRotation((event->x()-yR)/5);
      xR = event->y();
      //yR = event->x();
     }
  }


void GLBox::place_RAZ()
  {
   for( int i=0 ; i<16 ; i++ ) piece_maison( i );
   updateGL();
  }


void GLBox::set_piece_pos( int piece, position p )
  {
   obj[piece]->set_pos( (float)(p.x)-1.5, (float)(p.y)-1.5 );
   updateGL();
  }


void GLBox::set_piece_maison( int piece )
  {
   piece_maison( piece );
   updateGL();
  }


void GLBox::piece_maison( int piece )
  {
   obj[piece]->set_pos( (piece/8)-5, (float)(piece%8)-4 );
  }


void GLBox::switch_mode( int mode )
  {
   this->mode = mode;
   switch( mode )
     {
      case MODE_NONE:
             glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
             glDisable( GL_TEXTURE_2D );
             glDisable(GL_LIGHTING);
             break;
      case MODE_FLAT:
             glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
             glDisable( GL_TEXTURE_2D );
             glEnable(GL_LIGHTING);
             break;
      case MODE_TEX_NEAR:
             glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
             glEnable( GL_TEXTURE_2D );
             glEnable(GL_LIGHTING);
             break;
      case MODE_TEX_LINEAR:
             glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
             glEnable( GL_TEXTURE_2D );
             glEnable(GL_LIGHTING);
             break;
     }
   updateGL();
  }


int GLBox::pick( GLint x, GLint y )
  {
   GLuint selectBuf[32];
   GLint hits;
   GLint viewport[4];

   glGetIntegerv(GL_VIEWPORT, viewport);

   glSelectBuffer(32, selectBuf);
   (void) glRenderMode(GL_SELECT);

   glInitNames();
   glPushName(0);

   glMatrixMode(GL_PROJECTION);

   glPushMatrix();
   glLoadIdentity();

   gluPickMatrix((GLdouble) x, (GLdouble) viewport[3]-y+viewport[1]*2, 1.0, 1.0, viewport);
   glOrtho(-1, 1, -1, 1, -10, 10);

   paintGL();

   glPopMatrix();

   glFlush();
   hits = glRenderMode(GL_RENDER);

   glMatrixMode( GL_MODELVIEW );

   GLuint names, *ptr = selectBuf;

   GLfloat z1, z2, min_z=9999;
   GLuint good_name=0;
   for ( int i = 0; i < hits; i++)
     {
      names = *ptr;
      ptr++;

      z1 = (GLfloat) *ptr/0x7fffffff;
      ptr++;

      z2 = (GLfloat) *ptr/0x7fffffff;
      ptr++;

      for ( unsigned int j = 0; j < names; j++)
        {
         if( z1 < min_z )
           {
            min_z = z1;
            good_name = *ptr;
           }
         ptr++;
        }
     }

   if( hits > 0 && good_name != 9999 ) return good_name;
   return -1;
  }
