#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <qapplication.h>
#include <qfiledialog.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qwhatsthis.h>
#include <qinputdialog.h>
#include <qdir.h>
#include <qvbox.h>

#include "interface.h"
#include "get_path.h"
#include "gettext.h"

#include "xpm/grille.xpmS"
#include "xpm/idea.xpm"
#include "xpm/filechargement.xpm"
#include "xpm/fileexit.xpm"
#include "xpm/filesave.xpm"
#include "xpm/filenew.xpm"
#include "xpm/filehelp.xpm"
#include "xpm/filewhat.xpm"
#include "xpm/fileredo.xpm"
#include "xpm/fileundo.xpm"
#include "xpm/zoom_plus.xpm"
#include "xpm/zoom_moins.xpm"
#include "xpm/logo_long.xpm"
#include "xpm/logo.xpm"
#include "xpm/rotateG.xpm"
#include "xpm/rotateD.xpm"
#include "xpm/inclineB.xpm"
#include "xpm/inclineH.xpm"
#include "xpm/filehumain.xpm"
#include "xpm/filemachine.xpm"
#include "xpm/filedemo.xpm"

Menu::Menu(): QMainWindow( 0, PACKAGE){
  p = NULL;
  l = NULL;
  estHumain = FALSE;
  w = new QWidget(this);
  QPushButton *bouton = new QPushButton("r-Katro",w);
  connect(bouton, SIGNAL(clicked()), SLOT(katroclique()));
  bouton->setPixmap( logo_long ); 
  QWhatsThis::add(bouton, _("Button <b>r-Katro</b> allow "
		  "to player to point \"I'm the winner\"."
		  "<p>This button is aviable only for play in human vs human.</p>") );

  timer_humain = new QTimer( this );
  connect( timer_humain, SIGNAL(timeout()), SLOT(temp_humain()) );
  timer_machine = new QTimer( this );
  connect( timer_machine, SIGNAL(timeout()), SLOT(temp_machine()) );
  timer_demo = new QTimer( this );
  connect( timer_demo, SIGNAL(timeout()), SLOT(temp_demo()) );
  timer_historique = new QTimer( this );
  connect( timer_historique, SIGNAL(timeout()), SLOT(temp_historique()) );
 


  //Insertion de l'objet openGL dans le widget
  plateau = new GLBox( w, "glbox"); 
  QWhatsThis::add(plateau, _("It's in this area that have game board. "
		  "In order to have more information, type F1 or clic on <img source=\"filehelp\">") );
  plateau->setMinimumSize( 80, 80 );
  QVBoxLayout* vlayout = new QVBoxLayout( w );
  vlayout->addWidget( plateau );
  vlayout->addWidget( bouton );

  //cration de la barre d'tat
  ma_bar = new QWidget(w);
  ma_bar->setMaximumHeight( 30 ); 
  QWhatsThis::add(ma_bar, _("This is the status bar. It give courant player "
		  "thus some informations") );

  contenu_barG = new QStatusBar(ma_bar);
  contenu_barG->setSizeGripEnabled ( FALSE );
  
  contenu_barD = new QStatusBar(ma_bar);
  QHBoxLayout *hlayout = new QHBoxLayout( ma_bar );
  
  hlayout->addWidget( contenu_barG );
  hlayout->addWidget( contenu_barD );
  vlayout->addWidget( ma_bar );
 
  //insertion du qwidget dans la fentre
  setCentralWidget( w );

  /***************************** Creation des boutons ****************************************/
  QPixmap saveIcon, exitIcon, newIcon, chargeIcon, helpIcon, whatIcon, redoIcon, undoIcon,
    zoom_plusIcon, zoom_moinsIcon, rotateGIcon, rotateDIcon, inclineBIcon, inclineHIcon, 
    humainIcon, machineIcon, demoIcon, ideaIcon;


  fileTools = new QToolBar( this, "barre d'outils" );
  fileTools->setVerticalStretchable( TRUE );

  rotateGIcon = QPixmap( rotateG );
  rotateDIcon = QPixmap( rotateD );
  inclineBIcon = QPixmap( inclineB );
  inclineHIcon = QPixmap( inclineH );
  newIcon = QPixmap( filenew );
  whatIcon = QPixmap( filewhat );

  humainIcon = QPixmap( filehumain );
  QToolButton * fileHumain
    = new QToolButton( humainIcon, _("human vs human"), QString::null,
		       this, SLOT(init_humain()), fileTools, "joueur humain vs joueur humain" );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filehumain", humainIcon );
  QWhatsThis::add(fileHumain, _("The button <img source=\"filehumain\"> "
		  "allow to do a human vs human game.") );

  machineIcon = QPixmap( filemachine );
  QToolButton * fileMachine  
    = new QToolButton( machineIcon, _("human vs computer"), QString::null,
		       this, SLOT(init_machine()), fileTools, "joueur humain vs machine" );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filemachine", machineIcon );
  QWhatsThis::add(fileMachine, _("Ths button <img source=\"filemachine\"> allow "
		  "to do a human vs computer game.") );

  demoIcon = QPixmap( filedemo );
  QToolButton * fileDemo  
    = new QToolButton( demoIcon, _("computer vs computer"), QString::null,
		       this, SLOT(init_demo()), fileTools, "machine vs machine" );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filedemo", demoIcon );
  QWhatsThis::add(fileDemo, _("The button <img source=\"filedemo\"> allow "
		  "to do a demonstration game, "
		  "it's a computer vs computer game") );

  saveIcon = QPixmap( filesave );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filesave", saveIcon );
  QToolButton * fileSave
    = new QToolButton( saveIcon, _("Save"), QString::null,
		       this, SLOT(save()), fileTools, "sauvegarde" );
  QWhatsThis::add(fileSave, _("The button <img source=\"filesave\">  allow "
		  "to save the game.") );

  chargeIcon = QPixmap( filechargement );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filechargement", chargeIcon );
  QToolButton * fileCharge
    = new QToolButton( chargeIcon, _("Load"), QString::null,
		       this, SLOT(charge()), fileTools, "chargement" );
  QWhatsThis::add(fileCharge, _("The button <img source=\"filechargement\"> allow "
		  "to load 2 human player game.") );

  zoom_plusIcon = QPixmap( zoom_plus );
  QMimeSourceFactory::defaultFactory()->setPixmap( "zoom_plus", zoom_plusIcon );
  QToolButton * Zoom_Plus
    = new QToolButton( zoom_plusIcon, _("Zoom in"), QString::null,
		       this, SLOT(agrandi()), fileTools, "agrandi" );
  QWhatsThis::add(Zoom_Plus, _("The button <img source=\"zoom_plus\"> allow "
		  "to zoom in board game.") );


  zoom_moinsIcon = QPixmap( zoom_moins );
  QMimeSourceFactory::defaultFactory()->setPixmap( "zoom_moins", zoom_moinsIcon);
  QToolButton * Zoom_Moins
    = new QToolButton( zoom_moinsIcon, _("Zoom out"), QString::null,
		       this, SLOT(diminu()), fileTools, "diminue" );
  QWhatsThis::add(Zoom_Moins, _("The button <img source=\"zoom_moins\"> allow "
		  "to zoom out in board game.") );

  undoIcon = QPixmap( fileundo );
  QMimeSourceFactory::defaultFactory()->setPixmap( "fileundo", undoIcon );
  QToolButton * fileUndo
    = new QToolButton( undoIcon, _("Undo"), QString::null,
		       this, SLOT(undo()), fileTools, "undo" );
  QWhatsThis::add(fileUndo, _("The button <img source=\"fileundo\"> allow "
		  "to undo last moves.") );

  redoIcon = QPixmap( fileredo );
  QMimeSourceFactory::defaultFactory()->setPixmap( "fileredo", redoIcon );
  QToolButton * fileRedo
    = new QToolButton( redoIcon, _("Redo"), QString::null,
		       this, SLOT(redo()), fileTools, "redo" );
  QWhatsThis::add(fileRedo, _("The button <img source=\"fileredo\"> allow "
		  "to redo last undo moves.") );

  helpIcon = QPixmap( filehelp );
  QMimeSourceFactory::defaultFactory()->setPixmap( "filehelp", helpIcon );
  QToolButton * fileHelp
    = new QToolButton( helpIcon, _("Help"), QString::null,
			 this, SLOT(aide()), fileTools, "aide" );
  QWhatsThis::add(fileHelp, _("The button <img source=\"filehelp\"> allow "
		  "to call help.") );

  ideaIcon = QPixmap( idea );
  QMimeSourceFactory::defaultFactory()->setPixmap( "idea", ideaIcon );
  QToolButton * fileIdea
    = new QToolButton( ideaIcon, _("Game help"), QString::null,
			 this, SLOT(aide_jeu()), fileTools, "aide de jeu" );
  QWhatsThis::add(fileIdea, _("The button <img source=\"idea\"> allow "
		  "to call computer to play one move in place of you.") );
  
  (void)QWhatsThis::whatsThisButton( fileTools );
 
  exitIcon = QPixmap( fileexit );
  QMimeSourceFactory::defaultFactory()->setPixmap( "fileexit", exitIcon );
  QToolButton * fileExit
    = new QToolButton( exitIcon, _("Quit"), QString::null,
		       this, SLOT(quitter()), fileTools, "quitter" );
  QWhatsThis::add(fileExit, _("The button <img source=\"fileexit\"> allow "
		  "to quit the game") );

  /****************************** barre de menu *******************************/

  //sous menu nouveau
  QPopupMenu *nouveau = new QPopupMenu( this );
  CHECK_PTR( nouveau );
  nouveau->insertTearOffHandle();
  nouveau->insertItem( humainIcon, _("&2 players"), this, SLOT( init_humain() ), CTRL+Key_2);
  nouveau->insertItem( machineIcon, _("&1 player"), this, SLOT( init_machine() ), CTRL+Key_1);
  nouveau->insertItem( demoIcon, _("&Demonstration"), this, SLOT( init_demo()), CTRL+Key_0);

  //menu jeu
  QPopupMenu *jeu = new QPopupMenu( this );
  CHECK_PTR( jeu );
  jeu->insertItem(newIcon, _("&New"),  nouveau);
  jeu->insertItem(saveIcon, _("&Save"), this, SLOT(save()), CTRL+Key_S );
  jeu->insertItem(saveIcon, _("Save &as"), this, SLOT(save_as()) );
  jeu->insertItem(chargeIcon, _("&Load"), this, SLOT(charge()),CTRL+Key_O );
  jeu->insertSeparator();
  jeu->insertItem( _("&Historical"), this, SLOT(historique()),CTRL+Key_H );
  jeu->insertSeparator();
  jeu->insertItem(undoIcon, _("&Undo"), this, SLOT(undo()), CTRL+Key_Z);
  jeu->insertItem(redoIcon, _("&Redo"), this, SLOT(redo()), CTRL+Key_R);
  jeu->insertSeparator();
  jeu->insertItem(exitIcon, _("&Quit"),  qApp, SLOT(quit()), CTRL+Key_Q );
  
  //sous menu theme
  QPopupMenu *theme = new QPopupMenu( this );
  CHECK_PTR( theme );
  theme->insertTearOffHandle();
  
  theme_clasID = theme->insertItem(_("&Classic"));
  theme->setAccel( Key_F9, theme_clasID );
  theme->connectItem( theme_clasID, this, SLOT(theme_clas()));

  theme_moderID = theme->insertItem(_("&Modern"));
  theme->setAccel( Key_F10, theme_moderID );
  theme->connectItem( theme_moderID, this, SLOT(theme_moder()));



  //menu configuration
  QPopupMenu *configuration = new QPopupMenu( this );
  CHECK_PTR( configuration );

  //initialisation du sous menu theme
  isTheme_clas = FALSE;
  isTheme_moder = FALSE;
  configuration->setCheckable( TRUE );
  isTheme_moder = !isTheme_moder;
  theme->setItemChecked( theme_moderID, isTheme_moder );

  configuration->insertItem( _("&Skin"), theme );
  configuration->insertItem( _("&Default"), this, SLOT(initialise()),Key_F12 );


 
 //sous menu zoom
  QPopupMenu *zoom = new QPopupMenu( this );
  CHECK_PTR( zoom );
  zoom->insertTearOffHandle();
  zoom->insertItem(zoom_plusIcon,_("Zoom &+"),this,SLOT(agrandi()), Key_Plus);
  zoom->insertItem(zoom_moinsIcon,_("Zoom &-"),this,SLOT(diminu()), Key_Minus);

  //sous menu rotation
  QPopupMenu *rotation = new QPopupMenu( this );
  CHECK_PTR( rotation );
  rotation->insertTearOffHandle();
  rotation->insertItem(rotateGIcon, _("Rotation &Left"),this,SLOT(rotationG()),Key_Left);
  rotation->insertItem(rotateDIcon, _("Rotation &Right"),this,SLOT(rotationD()),Key_Right);
  rotation->insertItem(inclineHIcon, _("Inclie &Up"),this,SLOT(inclinaison_haute()),Key_Up);
  rotation->insertItem(inclineBIcon, _("Include &Down"),this,SLOT(inclinaison_basse()),Key_Down);

  //sous menu mode
  QPopupMenu *mode = new QPopupMenu( this );
  CHECK_PTR( mode );
  mode->insertTearOffHandle();
  
  file_ferID = mode->insertItem(_("&Line"));
  mode->setAccel( Key_F2, file_ferID );
  mode->connectItem( file_ferID, this, SLOT(affiche_file_fer()));

  uniID = mode->insertItem(_("&Smooth"));
  mode->setAccel( Key_F3, uniID );
  mode->connectItem( uniID, this, SLOT(affiche_uni()));

  tex_procheID = mode->insertItem(_("Texture &near"));
  mode->setAccel( Key_F4, tex_procheID );
  mode->connectItem( tex_procheID, this, SLOT(affiche_tex_proche()));

  tex_linID = mode->insertItem(_("Texture &linear"));
  mode->setAccel( Key_F5, tex_linID );
  mode->connectItem( tex_linID, this, SLOT(affiche_tex_lin()));

  //menu affichage
  QPopupMenu* affichage = new QPopupMenu( this );
  CHECK_PTR( affichage );
  
  //initialisation du sous menu mode
  isFile_fer = FALSE;
  isUni = FALSE;
  isTex_proche = FALSE;
  isTex_lin = FALSE;
  affichage->setCheckable( TRUE );
  isTex_proche = !isTex_proche;
  mode->setItemChecked( tex_procheID, isTex_proche );
  
  affichage->insertItem( _("&Mode"), mode );
  affichage->insertItem( _("&Zoom"), zoom );
  affichage->insertItem( _("&Rotation"), rotation );

  //menu aide
  QPopupMenu *help = new QPopupMenu( this );
  CHECK_PTR( help );
  help->insertItem( helpIcon, _("&Help"), this, SLOT(aide()), Key_F1);
  help->insertItem( ideaIcon, _("&Game help"), this, SLOT(aide_jeu()), Key_Asterisk);
  help->insertItem( _("&About r-Katro"), this, SLOT(about()), CTRL+Key_A );
  help->insertSeparator();
  help->insertItem( whatIcon, _("&What is it?"), this, SLOT(whatsThis()), SHIFT+Key_F1 );

  mon_menu = new QMenuBar( this );
  CHECK_PTR( mon_menu );
  mon_menu->insertItem( _("&Game"), jeu );
  mon_menu->insertItem( _("&Configuration"), configuration );
  mon_menu->insertItem( _("&Display"), affichage );
  mon_menu->insertSeparator();
  mon_menu->insertItem( _("&?"), help );
  mon_menu->setSeparator( QMenuBar::InWindowsStyle );
 
  setMinimumSize( 600, 450 );
  contenu_barG->message( _("Ready") );
}

Menu::~Menu()
{
  if( p != NULL )
    delete p;
}


/************************** les slots ****************************************/
void Menu::rotationG()
{
  contenu_barD->message( _("rotation left"), 2000 );
  plateau->setZRotation( -10 );
}

void Menu::rotationD()
{
  contenu_barD->message( _("rotation right"), 2000 );
  plateau->setZRotation( 10 );
}

void Menu::inclinaison_haute()
{
  contenu_barD->message( _("incline up"), 2000 );
  plateau->setXRotation( -10 );
}

void Menu::inclinaison_basse()
{
  plateau->setXRotation( 10 );
  contenu_barD->message( _("incline down"), 2000 );
}

void Menu::agrandi()
{
  plateau->set_scale( 0.05 );
  contenu_barD->message( _("zoom in"), 2000 );
}

void Menu::diminu()
{
  plateau->set_scale( -0.05 );
  contenu_barD->message( _("zoom out"), 2000 );
}

void Menu:: quitter()
{
  if( p != NULL )
    delete p;
  p = NULL; 
  QApplication::exit(0);
}

void Menu::affiche_file_fer()
{
  if(isFile_fer == FALSE)
    {
      contenu_barD->message( _("mode line"), 2000 );
      plateau->switch_mode( MODE_NONE );
      isFile_fer = !isFile_fer;
      isUni = FALSE;
      isTex_proche = FALSE;
      isTex_lin = FALSE;
      mon_menu->setItemChecked( file_ferID, isFile_fer );
      mon_menu->setItemChecked( uniID, isUni );
      mon_menu->setItemChecked( tex_procheID, isTex_proche );
      mon_menu->setItemChecked( tex_linID, isTex_lin );
    }
}

void  Menu::affiche_tex_lin()
{
  if(isTex_lin == FALSE)
    {
      contenu_barD->message( _("mode texture linear"), 2000 );
      plateau->switch_mode( MODE_TEX_LINEAR );
      isTex_lin = !isTex_lin;
      isFile_fer = FALSE;
      isUni = FALSE;
      isTex_proche = FALSE;
      mon_menu->setItemChecked( file_ferID, isFile_fer );
      mon_menu->setItemChecked( uniID, isUni );
      mon_menu->setItemChecked( tex_procheID, isTex_proche );
      mon_menu->setItemChecked( tex_linID, isTex_lin );
    }
}

void  Menu::affiche_tex_proche()
{
  if(isTex_proche == FALSE)
    {
      contenu_barD->message( _("mode texture near"), 2000 );
      plateau->switch_mode( MODE_TEX_NEAR );
      isTex_proche = !isTex_proche;
      isTex_lin = FALSE;
      isFile_fer = FALSE;
      isUni = FALSE;
      mon_menu->setItemChecked( file_ferID, isFile_fer );
      mon_menu->setItemChecked( uniID, isUni );
      mon_menu->setItemChecked( tex_procheID, isTex_proche );
      mon_menu->setItemChecked( tex_linID, isTex_lin );
    }
}

void  Menu::affiche_uni()
{
  if(isUni == FALSE)
    {
      contenu_barD->message( _("smooth"), 2000 );
      plateau->switch_mode( MODE_FLAT );
      isUni = !isUni;
      isTex_lin = FALSE;
      isFile_fer = FALSE;
      isTex_proche = FALSE;
      mon_menu->setItemChecked( file_ferID, isFile_fer );
      mon_menu->setItemChecked( uniID, isUni );
      mon_menu->setItemChecked( tex_procheID, isTex_proche );
      mon_menu->setItemChecked( tex_linID, isTex_lin );
    }
}

void Menu::initialise()
{
  contenu_barD->message( _("incline display"), 2000 );
  plateau->set_RAZ();
}

void Menu::temp_humain()
{ 
  timer_humain->start( 100, FALSE ); 
  QString msg( _("Turn to player ") );
  msg += QString::number( p->joueurNum()+1 );
  contenu_barG->message( msg );
  if( p->alignementKatro()==FALSE )
    if(p->finPartie()) 
      ordi_fin( _("Game is null") );
}

void Menu::init_humain()
{  
  if( p != NULL )  
    delete p;
  p = new principal(0);
  timer_machine->stop();
  timer_demo->stop();
  estHumain = TRUE;
  contenu_barD->message( _("human vs human"), 2000 );
  plateau->place_RAZ();
  plateau->set_move( p ); 
  temp_humain();
}

void Menu::temp_machine()
{
  timer_machine->start( 100, FALSE );
  if( p->joueurNum()==1 )
    contenu_barG->message( _("Turn to computer") );
  else
    contenu_barG->message( _("Turn to player") );
  int pnumAI;
  position posAI;
  if( p->tour_de_AI( pnumAI, posAI ) )
      plateau->set_piece_pos( pnumAI, posAI );
  
  //test de fin de partie
  if( p->alignementKatro() )
    {
      if(p->joueurNum()==1)
	ordi_fin( _("The computer win") );
      else
	{
	  ordi_fin( _("The player win") );
	}
    }
  else
    {
      if(p->finPartie())
	ordi_fin( _("Game is null") );
    }
}

void Menu::init_machine()
{
  if( p != NULL )
    delete p;
  p = new principal(1);
  timer_humain->stop();
  timer_demo->stop();
  contenu_barD->message( _("human vs computer"), 2000 );
  plateau->place_RAZ();
  plateau->set_move( p );
  temp_machine();
}

void Menu::temp_demo()
{ 
  timer_demo->start( 100, FALSE );
  int pnumAI;
  position posAI;
  if(p->tour_de_AI( pnumAI, posAI ))
     plateau->set_piece_pos( pnumAI, posAI );
  
  if( (p->game)->alignementKatro() )
    {
      ordi_fin( _("The computer win") );
    }
  else
    {
      if(p->finPartie())
	ordi_fin( _("Game is NULL") );
    }
}

void Menu::init_demo()
{
  if( p != NULL )  
    delete p;
  p = new principal(2);
  timer_humain->stop();
  timer_machine->stop();
  contenu_barD->message( _("computer vs computer"), 2000 ); 
  contenu_barG->message( _("Turn to computer") );
  plateau->place_RAZ();
  plateau->set_move( p );
  temp_demo();
}

void Menu::save()
{
  if ( filename.isEmpty() ) {
    save_as();
    return;
  }
  if(p != NULL)
    {
      QFileInfo fi( filename );
      if ( fi.extension() != "r-katro" )
	filename += ".r-katro";
      if(p->sauver( filename ))
	{
	  contenu_barD->message( _("file saved"), 2000 );
	  return;
	}
    }
  fprintf( stderr, _("Game/File not saved\n") );
  contenu_barD->message( _("file not saved"), 2000 );
}

void Menu::save_as()
{
  QString fn = QFileDialog::getSaveFileName( QString::null, filtre(), this );
  if ( !fn.isEmpty() ) {
	filename = fn;
	save();
    } 
  else
    {
      fprintf( stderr, _("Game/File not saved\n") );
      contenu_barD->message( _("file not saved"), 2000 );
    }
}

void Menu::charge()
{
  principal *tmp = p;
  p = new principal(0);

  QString fn = QFileDialog::getOpenFileName( QString::null, filtre(), this);
  if ( !fn.isEmpty() )
    {
	  if(p->recupererpartie( fn ))
	    {
	      if( tmp != NULL )
		delete tmp;
	      timer_humain->stop();
	      timer_machine->stop();
	      timer_demo->stop();
	      plateau->set_move( p );
	      contenu_barD->message( _("file open"), 2000 );
	      estHumain = TRUE;
	      plateau->move_trans( FALSE );
	      plateau->place_RAZ();
	      for(int i=0;i<4;i++)
		for(int j=0;j<4;j++)
		  if((p->game)->plat[i][j]!=NULL)
		    {
		      position pos = {i,j};
		      plateau->set_piece_pos( ((p->game)->plat[i][j])->num, pos );
		    }
	      if((p->game)->p_cour!=NULL)
		{
		  position pos = {-1,-1};
		  plateau->set_piece_pos( ((p->game)->p_cour)->num, pos );
		}
	      plateau->move_trans( TRUE );
	      temp_humain();
	      return;
	    }
    }
  delete p;
  p = tmp;
  fprintf( stderr, _("Game/File not loaded\n") );
  contenu_barD->message( _("file not loaded"), 2000 );
}

void Menu::aide()
{
  contenu_barD->message( _("Help selected"), 2000 );
  system(QString("r-katro.helpviewer ")+get_path(_("doc/help.en.html"))+" &");
}

void Menu::aide_jeu()
{
  contenu_barD->message( _("Game help selected"), 2000 );
  int pnumAI;
  position posAI;
  if(p != NULL)
    {
      if(p->finPartie()==FALSE)
	{
	  if(p->aide( pnumAI, posAI ))
	      plateau->set_piece_pos( pnumAI, posAI );
	}
    }
}

void Menu::about()
{
  contenu_barD->message( _("About selected"), 2000 );
  QMessageBox *apropos = new QMessageBox( this, _("About r-Katro") );
  QPixmap qIcon = QPixmap( logo );
  apropos->setIconPixmap ( qIcon );
  apropos->setText( QString(_("<p><b>r-Katro</b></p>"
                "<p>This game was build originally by the r-Katro Team:</p>"))+
                "<ul><li>"+
                _("Programmer")+
                    "<ul>"
                    "<li>ARBYIG Basil</li>"
                    "<li>CHAMEAUX Damien</li>"
                    "<li>RODRIGO Frdric (maintainer <a href=\"mailto:f.rodrigo@tuxfamily.org\">f.rodrigo@tuxfamily.org</a>)</li>"
                    "<li>SERRA Cline</li>"
                    "</ul></li>"
                "<ul><li>"+
                _("Designer")+
                   "<ul>"
                   "<li>BIOUSSE Yannick</li>"
                   "</ul></li>"
                "<ul><li>"+
                _("Thank to")+
                   "<ul>"
                   "<li>BOIRON Vronique</li>"
                   "<li>CAVAIGNAC Sbastien</li>"
                   "<li>LAMASSE Xavier</li>"
                   "</ul></li>"
                "</ul>" );
  apropos->setCaption( _("About r-Katro") );
  apropos->exec();
}

void Menu::undo()
{
  if( p != NULL)
    {
      int numero;
      position pos;
	  if( p->undo(pos, numero ) )
	    {
	      if(numero<0) 
		// piece sur le plateau  mettre en dehors de plateau
		{
		  numero = -numero-1;
		  plateau->set_piece_pos( numero, pos );
		}
	      else
		// piece en dehors de plateau qui est donn
		//  remettre parmi les pieces libres
		{
		  plateau->set_piece_maison( numero );
		}
	      
	      contenu_barD->message( _("undo"), 2000 );     
	      return;     
	    }
    }
  contenu_barD->message( _("no undo"), 2000 );
}

void Menu::redo()
{
  if( p != NULL )
    {
      int numero;
      position pos;
      if( p->redo( pos,numero ) )
	{
	  if(numero<0)
	    numero = -numero-1;
	  plateau->set_piece_pos( numero, pos );
	  contenu_barD->message( _("redo"), 2000 );
	  return;
	}
    }
  contenu_barD->message( _("no redo"), 2000 );
}

void Menu::temp_historique()
{
  timer_historique->start( 100, FALSE );
  if( p != NULL )
    {
      if(etape_actuel != p->game->cour)
	{
	  if(etape_actuel < p->game->cour)
	    {
	      for(int i = etape_actuel; i< p->game->cour; i++)
	    
		{
		  QPixmap grilleIcon = QPixmap( grille[(p->game)->tetape[i].pos.x][(p->game)->tetape[i].pos.y] );
		  int cr = (p->game)->tetape[i].pnum & 1;
		  int ro = (p->game)->tetape[i].pnum & 2;
		  int gr = (p->game)->tetape[i].pnum & 8;
		  int bl = (p->game)->tetape[i].pnum & 4;
		  QString nom_piece("");
		  nom_piece += ( cr? _("hollow ") : _("solid ") );
		  nom_piece += ( ro? _("round ")  : _("square ") );
		  nom_piece += ( gr? _("small ")  : _("large ") );
		  nom_piece += ( bl? _("white ")  : _("black ")  );
		  l->insertItem( grilleIcon, QString::fromUtf8( _("played -> ") ) + nom_piece);
		}
	    }
	  else
	    for(int i = etape_actuel; i>= p->game->cour; i--)
	    l->removeItem(i);
	  
	  etape_actuel =  p->game->cour;
	}
    }
}

void Menu::historique()
{
  if(p != NULL)
    {
      QVBox *tab = new QVBox( this );
      tab->setMinimumSize( 400, 200 );
      l = new QListBox( tab );
      l->setFocusPolicy( QWidget::StrongFocus );
      etape_actuel=0;
      temp_historique();
      message_histo = new QTabDialog( this, _("Message of end") );
      message_histo->addTab( tab, _("Historical") );
      message_histo->setOkButton( _("Close") );
      message_histo->show(); 
    }
}



void Menu::katroclique()
{
  if( p != NULL )
    {
      if( estHumain )
	{
	  if( (p->alignementKatro())==FALSE )
	    ordi_fin ( _("The player ") + QString::number( p->joueurNum()+1 ) + _(" have lost") );
	  else
	    {
	      ordi_fin ( _("The player ") + QString::number( p->joueurNum()+1 ) + _(" have win") );
	    }
	}
    }
}

void Menu::theme_clas()
{
  if( isTheme_clas == FALSE )
    {
      contenu_barD->message( _("classic theme"), 2000 );
      plateau->set_theme( "classic" );
      isTheme_clas = !isTheme_clas;
      isTheme_moder = FALSE;
      mon_menu->setItemChecked( theme_moderID, isTheme_moder );
      mon_menu->setItemChecked( theme_clasID, isTheme_clas );
    }
}

void Menu::theme_moder()
{
  if( isTheme_moder == FALSE )
    {
      contenu_barD->message( _("modern theme"), 2000 );
      plateau->set_theme( "modern" );
      isTheme_moder = !isTheme_moder;
      isTheme_clas = FALSE;
      mon_menu->setItemChecked( theme_moderID, isTheme_moder );
      mon_menu->setItemChecked( theme_clasID, isTheme_clas );
    }
}



/********************* fonctions prives d'interface.cc ********************/

QString Menu::filtre()
{
  QString f = tr(_("r-Katro files (*katro)")) + ";;" + tr(_("All files (*)"));
  return f;
}

void Menu::ordi_fin( QString quoi )
{
  if( p != NULL )
    {
      timer_humain->stop();
      timer_demo->stop();
      timer_machine->stop(); 
      contenu_barG->message( quoi ); 
      QMessageBox *message = new QMessageBox( this, _("Message of end") );
      QPixmap qIcon = QPixmap( logo );
      message->setIconPixmap ( qIcon );
      message->setText( "<b>" + quoi + "</b>" );
      message->exec();
      if(l!=NULL) 
	l->clear();
      estHumain = FALSE;
      plateau->set_move( NULL );
      plateau->set_RAZ();
      contenu_barG->message( _("Ready") ); 
      delete p;
      p = NULL;
    }
}
