#include "obj3d.h"
#include <libgen.h>
#include "gettext.h"
#include "get_path.h"

static GLuint IdTex[3];
static GLuint  piece_objet[8] = {9999,9999,9999,9999,9999,9999,9999,9999};
static GLfloat piece_depth[8];


void obj3d::set_new_theme()
  {
   for( int i=0 ; i<8 ; i++ )
      piece_objet[i] = 9999;
  }


obj3d::obj3d( char *theme, char *file )
  {
   FILE *piece;
   piece = fopen( get_path(file), "r" );
   if( piece == NULL ) 
     {
      printf( _("Error with file %s\n"), file );
      exit(1);
     }

   char tmp[30];
   sprintf( tmp, "theme/%s/%%i", theme );
   if( sscanf( file, tmp, &name ) != 1 ) name = 9999;

   char obj[20], map[20], dir_obj[256];
   float norm_taille;
   if( fscanf( piece, "%19s%19s%f", obj, map, &norm_taille ) != 3 ) printf( _("Error of syntaxe in %s\n"), file );
   fclose( piece );
   printf( _("Loading %s\n"), file);

   GLfloat size[3];
   int p_name = ( name<8 )? ( (name<4)?name:name-4) : ( (name<12)?name-4:name-8 );

  if( name == 9999 || piece_objet[ p_name ] == 9999 )
    {
     snprintf( dir_obj, 255, "%s/%s", dirname(get_path(file)), obj );
     GLMmodel *m = glmReadOBJ( dir_obj );
     if( m == NULL ) printf( _("Error with file %s\n"), dir_obj );

     glmDimensions( m, size );
     glmScale( m, norm_taille/size[2] );
     depth = norm_taille;
     if( name != 9999 ) piece_depth[ p_name ] = depth;

     glmFacetNormals(m);
     glmVertexNormals(m,90);
     glmLinearTexture( m );

     glmDimensions( m, size );

     object = glmList( m, GLM_SMOOTH | GLM_TEXTURE );
     if( name != 9999 ) piece_objet[ p_name ] = object;
     glmDelete(m);
    }
   else
    {
     object = piece_objet[ p_name ];
     depth  = piece_depth[ p_name ];
    }

   if( strcmp("white",map)==0 )
     {
      set_couleur(0,0,1);
      texture = 0;
     }
   else if( strcmp("black",map)==0 )
     {
      set_couleur(1,0,0);
      texture = 1;
     }
   else if( strcmp("board",map)==0 )
     {
      set_couleur(1,.63,.27);
      texture = 2;
      depth = size[1];
     }
   else
    {
     printf( _("Error of piece type \"%s\"\n"), map );
     texture = 0;
    }  
  }

obj3d::~obj3d( void )
  {
   glDeleteLists( object, 1 );
  }

void obj3d::set_pos( GLfloat x, GLfloat y, GLfloat z )
  {
   this->x = x;
   this->y = y;
   this->z = z;
  }

void obj3d::set_pos( GLfloat x, GLfloat y )
  {
   if( texture != 2 )
     set_pos( x, y, depth/2 );
    else
     {
      set_pos( 0, depth/2, 0 );
     }
  }

void obj3d::set_pos()
  {
   set_pos( x, y );
  }


void obj3d::get_pos( GLfloat &x, GLfloat &y )
  {
   x = this->x;
   y = this->y;
  }


void obj3d::set_texture( GLuint i )
  {
   texture = i;
  }


void obj3d::set_couleur( GLfloat r_, GLfloat g_, GLfloat b_ )
  {
   r = r_;
   g = g_;
   b = b_;
  }


void obj3d::aff( int mode )
  {
   glPushMatrix();

   switch( mode )
     {
      case 0: {
               glColor3f(r,g,b);
              }
         break;
      case 1: {
               GLfloat Mspec[]={r,g,b};
               float specular[] = {0,0,0,1}; // spcifiquation du mat
               float difuse[] = {r,g,b,0};
               glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
               glMaterialfv(GL_FRONT, GL_DIFFUSE, difuse );
               glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,Mspec);
               /*
               GLfloat Mshiny=128; // briallance
               glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS,Mshiny);
               */
              }
         break;
      case 2:
              {
               float difuse[] = {1,1,1,0};
               glMaterialfv(GL_FRONT, GL_DIFFUSE, difuse );
               glBindTexture(GL_TEXTURE_2D,IdTex[texture]);
               glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
               glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
              }
         break;
      case 3:
              {
               float difuse[] = {1,1,1,0};
               glMaterialfv(GL_FRONT, GL_DIFFUSE, difuse );
               glBindTexture(GL_TEXTURE_2D,IdTex[texture]);
               glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
               glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
              }
         break;
     }

   glLoadName(name);

   if( texture == 2 )
     {
      if( mode >= 2 )
       {
        glRotatef( -90, 1, 0, 0 );
        glTranslated( x, y, z );
        glCallList( object );
       }
     }
    else
     {
      glTranslated( x, y, z );
      glCallList( object );
     }

   if( texture == 2 &&  mode <= 1 )
     {
      if( mode == 1 ) glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      glColor3f(0,0,0);
      glBegin( GL_QUADS );
        glVertex3f( -2, -2, 0 );        glVertex3f(  2, -2, 0 );        glVertex3f(  2,  2, 0 );        glVertex3f( -2,  2, 0 );
        glVertex3f( -1, -2, 0 );        glVertex3f(  1, -2, 0 );        glVertex3f(  1,  2, 0 );        glVertex3f( -1,  2, 0 );
        glVertex3f( -2, -1, 0 );        glVertex3f(  2, -1, 0 );        glVertex3f(  2,  1, 0 );        glVertex3f( -2,  1, 0 );
      glEnd();
      glBegin( GL_LINES );
        glVertex3f(  0, -2, 0 );        glVertex3f(  0,  2, 0 );        glVertex3f(  2,  0, 0 );        glVertex3f( -2,  0, 0 );
      glEnd();
      if( mode == 1 )glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
     }


   glPopMatrix();
  }


void obj3d::SET_TEXTURE( QImage b, QImage n, QImage p  )
  {
   glGenTextures(3,IdTex);

   glBindTexture(GL_TEXTURE_2D,IdTex[0]);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D,0,3,b.width(),b.height(),0,GL_RGBA,GL_UNSIGNED_BYTE,b.bits());

   glBindTexture(GL_TEXTURE_2D,IdTex[1]);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D,0,3,n.width(),n.height(),0,GL_RGBA,GL_UNSIGNED_BYTE,n.bits());

   glBindTexture(GL_TEXTURE_2D,IdTex[2]);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D,0,3,p.width(),p.height(),0,GL_RGBA,GL_UNSIGNED_BYTE,p.bits());
  }

